from zope.interface import implements
from zope.component import adapts
from Products.ATContentTypes.interface.image import IATImage
from Products.ImageEditor.interfaces.imageeditor import IImageEditorAdapter
from Products.ImageEditor.interfaces.unredostack import IUnredoStack
from PIL import Image, ImageFilter, ImageEnhance
from cStringIO import StringIO
from Products.ImageEditor.utils.conditions import precondition, between

class ImageEditorAdapter(object):
    implements(IImageEditorAdapter)
    adapts(IATImage)
    
    def __init__(self, image):
        self.image = image
        #adapter to handle redo and undo
        self.unredo = IUnredoStack(image)
        
    def undo(self):
        self.unredo.undo()
        
    def redo(self):
        self.unredo.redo()
        
    def clearEdits(self):
        self.unredo.clearStack()

    def saveEdit(self):
        image_data = self.unredo.getCurrent()
        
        field = self.image.getField('image')
        mimetype = field.getContentType(self.image)
        filename = field.getFilename(self.image)
        
        # because AT tries to get mimetype and filename from a file like
        # object by attribute access I'm passing a string along
        self.image.setImage(
            image_data, 
            mimetype=mimetype,
            filename=filename, 
            refresh_exif=False
        )
        
        #should I clear???  Maybe you should still be able to undo...              
        self.unredo.clearStack(image_data)
        
    def getOriginalImage(self):
        return Image.open(StringIO(self.unredo.stack[0]))
        
    def getCurrentImage(self):
        return Image.open(StringIO(self.unredo.getCurrent()))
       
    def setImage(self, image, format="JPEG", quality=None):
        """
        Setting the image just adds to the unredo stack...
        """
        image_data = StringIO()
        
        if quality:
            image.save(image_data, format, quality=quality)
        else:
            image.save(image_data, format)
        
        self.unredo.do(image_data.getvalue())
       
    def getCurrentImageData(self):
        return self.unredo.getCurrent()
       
    def getCurrentImageInfo(self):
        data = self.getCurrentImageData()
        bsize = len(data)
        bsize = bsize/1024

        size_descriptor = 'kb'
        if bsize > 1024:
            bsize = bsize/1024.0
            size_descriptor = 'mb'

        size = self.getCurrentImage().size

        width = size[0]
        height = size[1]

        return {
            'size': bsize,
            'width': width,
            'height': height,
            'sizeformatted': "Size: %s%s" % (bsize, size_descriptor)
        }
       
    def rotateLeft(self):
        original = self.getCurrentImage()
        image = original.rotate(90)

        self.setImage(image, original.format)
        
    def rotateRight(self):
        original = self.getCurrentImage()
        image = original.rotate(270)

        self.setImage(image, original.format)
        
    def flipOnVerticalAxis(self):
        original = self.getCurrentImage()
        image = original.transpose(Image.FLIP_TOP_BOTTOM)

        self.setImage(image, original.format)
        
    def flipOnHorizontalAxis(self):
        original = self.getCurrentImage()
        image = original.transpose(Image.FLIP_LEFT_RIGHT)

        self.setImage(image, original.format)
        
    def blur(self, amount):
        image = self.getCurrentImage()
        fmt = image.format
        for x in range(0, amount):
            image = image.filter(ImageFilter.BLUR)
            
        self.setImage(image, fmt)
        
#    @precondition(between(0, 100))
    def compress(self, amount):
        image = self.getCurrentImage().convert('RGB') # if it is a png, convert it...
        self.setImage(image, quality=amount)
        
#    @precondition(between(0.0, 2.0))
    def contrast(self, amount):
        image = self.getCurrentImage()
        enhancer = ImageEnhance.Contrast(image)
        newImage = enhancer.enhance(amount)

        self.setImage(newImage, image.format)
        
#    @precondition(between(0.0, 2.0))
    def brightness(self, amount):
        """
        amount should be between 0.0 and 2.0
        1.0 being the original image
        """
        image = self.getCurrentImage()
        enhancer = ImageEnhance.Brightness(image)
        #can enhance from 0.0-2.0, 1.0 being original image
        newImage = enhancer.enhance(amount)

        self.setImage(newImage, image.format)
        
    def sharpen(self, amount):
        image = self.getCurrentImage()
        enhancer = ImageEnhance.Sharpness(image)
        newImage = enhancer.enhance(amount)

        self.setImage(newImage, image.format)
    
    def resize(self, width, height):
        image = self.getCurrentImage()
        format = image.format
        size=(width, height)
        new_image = image.resize(size, Image.ANTIALIAS)
        
        self.setImage(new_image, image.format)
        
    def crop(self, tlx, tly, brx, bry):
        image = self.getCurrentImage()
        format = image.format
        box = (tlx, tly, brx, bry)
        new_image = image.crop(box=box)
        new_image.load()
        
        self.setImage(new_image, image.format)
        
    def dropshadow(self):
        """
        Code taken from http://code.activestate.com/recipes/474116/
        """
        image = self.getCurrentImage()
        offset = (5,5)
        background=0xffffff
        shadow=0x444444
        border=8
        iterations=3
        format = image.format
        
        # Create the backdrop image -- a box in the background colour with a 
        # shadow on it.
        totalWidth = image.size[0] + abs(offset[0]) + 2*border
        totalHeight = image.size[1] + abs(offset[1]) + 2*border
        back = Image.new(image.mode, (totalWidth, totalHeight), background)

        # Place the shadow, taking into account the offset from the image
        shadowLeft = border + max(offset[0], 0)
        shadowTop = border + max(offset[1], 0)
        back.paste(shadow, [shadowLeft, shadowTop, shadowLeft + image.size[0], 
        shadowTop + image.size[1]] )

        # Apply the filter to blur the edges of the shadow.  Since a small kernel
        # is used, the filter must be applied repeatedly to get a decent blur.
        n = 0
        while n < iterations:
            back = back.filter(ImageFilter.BLUR)
            n += 1

        # Paste the input image onto the shadow backdrop  
        imageLeft = border - min(offset[0], 0)
        imageTop = border - min(offset[1], 0)
        back.paste(image, (imageLeft, imageTop))
        
        self.setImage(back, format)
        
        


from zope.interface import Interface, Attribute

class IUnredoStack(Interface):
    
    pos = Attribute("""Position of the stack""")
    stack = Attribute("""Actual stack...""")
    
    def do(self, value):
        """
        performing the action
        """
        
    def getCurrent(self):
        """
        Returns the current element in stack
        """
        
    def undo(self):
        """
        Undo action
        """
        
    def canUndo(self):
        """
        Returns boolean if they can undo
        """
        
    def redo(self):
        """
        redo last action
        """
        
    def canRedo(self):
        """
        Returns boolean if they can redo
        """
        
    def clearStack(self):
        """
        Clears the stack
        """
    



from Products.ImageEditor.interfaces.unredostack import IUnredoStack
from zope.interface import implements
from zope.component import adapts
from Products.ATContentTypes.interface.image import IATImage
from PIL import Image
from cStringIO import StringIO

class UnredoStack(object):
    """
    see interfaces.py for info
    """
    implements(IUnredoStack)
    adapts(IATImage)

    def __init__(self, image):
        self.image = image
        
        if not hasattr(image, 'stack_pos'):
            self.pos = 0
        if not hasattr(image, 'unredostack'):
            self.stack = [image.data]
        
    def get_pos(self):
        return self.image.stack_pos
    def set_pos(self, value):
        self.image.stack_pos = value
    pos = property(get_pos, set_pos)

    def get_stack(self):
        return self.image.unredostack
    def set_stack(self, value):
        self.image.unredostack = value
    stack = property(get_stack, set_stack)

    def do(self, value):
        if self.canRedo():
            #clear top
            for item in self.stack[(self.pos+1):len(self.stack)]:
                self.stack.remove(item)

        self.stack.append(value)
        self.pos = self.pos + 1

    def getCurrent(self):
        return self.stack[self.pos]

    def undo(self):
        self.pos = self.pos - 1

    def canUndo(self):
        return self.pos > 0

    def redo(self):
        self.pos = self.pos + 1

    def canRedo(self):
        return self.pos + 1 < len(self.stack)
        
    def clearStack(self, bottom=None):
        self.pos = 0
        
        if bottom is None:
            bottom = self.image.data
        
        self.stack = [bottom]


from Products.PloneTestCase.PloneTestCase import PloneTestCase
from zope.testing import doctestunit
from zope.component import testing
from Testing import ZopeTestCase as ztc
from Products.ImageEditor.tests.base import ImageEditorTestCase
from cStringIO import StringIO
import zope.app.publisher.browser
from Products.Five.testbrowser import Browser
from PIL import Image, ImageFilter, ImageEnhance
from cStringIO import StringIO

class TestImageEdits(ImageEditorTestCase):
    """
    Test the edit actions on images
    """
        
    def test_dropShadow(self):
        imageEditor = self.getImageEditorAdapter()
        imageEditor.dropshadow()
        self.imagesEqual('dropshadow.jpg', imageEditor)
        
    def test_sharpening(self):
        imageEditor = self.getImageEditorAdapter()        
        imageEditor.sharpen(1.5)
        self.imagesEqual('1.5sharpening.jpg', imageEditor)
        
    def test_quality(self):
        imageEditor = self.getImageEditorAdapter()        
        imageEditor.compress(50)
        #testing this isn't working
        #self.imagesEqual('50percentquality.jpg', imageEditor)
                
    def test_brightness(self):
        imageEditor = self.getImageEditorAdapter()
        imageEditor.brightness(.75*2.0)
        self.imagesEqual('75percentbrightness.jpg', imageEditor)
        
    def test_contrast(self):
        imageEditor = self.getImageEditorAdapter()
        imageEditor.contrast(.75*2.0)
        self.imagesEqual('75percentcontrast.jpg', imageEditor)
                
    def test_horizontalFlip(self):
        imageEditor = self.getImageEditorAdapter()
        imageEditor.flipOnHorizontalAxis()
        self.imagesEqual('horizontalflip.jpg', imageEditor)
        
    def test_verticalFlip(self):
        imageEditor = self.getImageEditorAdapter()
        imageEditor.flipOnVerticalAxis()
        self.imagesEqual('verticalflip.jpg', imageEditor)        
        
    def test_blur(self):
        imageEditor = self.getImageEditorAdapter()
        imageEditor.blur(2)
        self.imagesEqual('level2blur.jpg', imageEditor)        
        
    def test_resize(self):
        imageEditor = self.getImageEditorAdapter()
        imageEditor.resize(390, 478)
        self.imagesEqual('resizeto390by478.jpg', imageEditor)        
        
    def testRotateLeft(self):
        imageEditor = self.getImageEditorAdapter()
        imageEditor.rotateLeft()
        self.imagesEqual('rotateleft.jpg', imageEditor)        
        
    def test_rotateRight(self):
        imageEditor = self.getImageEditorAdapter()
        imageEditor.rotateRight()
        self.imagesEqual('rotateright.jpg', imageEditor)        
        
class TestAdapter(ImageEditorTestCase):
    """
    Test the edit actions on images
    """
        
    def test_undo(self):
        editor = self.getImageEditorAdapter()
        
        original_image = editor.getCurrentImageData()
        editor.dropshadow()
        self.failUnless(original_image != editor.getCurrentImageData())
        
        editor.undo()
        self.failUnless(original_image == editor.getCurrentImageData())
        
    def test_redo(self):
        editor = self.getImageEditorAdapter()
        
        original_image = editor.getCurrentImageData()
        editor.dropshadow()
        
        edited_image = editor.getCurrentImageData()
        self.failUnless(original_image != edited_image)
        
        editor.undo()
        self.failUnless(original_image == editor.getCurrentImageData())
        
        editor.redo()
        self.failUnless(edited_image == editor.getCurrentImageData())
        
    def test_clearEdits(self):
        editor = self.getImageEditorAdapter()
        
        original_image = editor.getCurrentImageData()
        
        editor.dropshadow()
        editor.rotateLeft()
        editor.rotateRight()
        
        editor.clearEdits()
        self.failUnless(original_image == editor.getCurrentImageData())
        
    def test_saveEdits(self):
        editor = self.getImageEditorAdapter()
        
        editor.dropshadow()
        editor.rotateLeft()
        
        current = editor.getCurrentImageData()
        editor.saveEdit()
        
        self.failUnless(len(editor.unredo.stack) == 1)
        self.failUnless(current == editor.getCurrentImageData())
        
    def test_getCurrentImage(self):
        editor = self.getImageEditorAdapter()
        
        editor.dropshadow()
        editor.rotateLeft()
        
        self.failUnless(editor.getCurrentImageData() == editor.unredo.getCurrent())
    
    def test_setImage(self):
        editor = self.getImageEditorAdapter()
        
        self.failUnless(len(editor.unredo.stack) == 1)
        
        editor.dropshadow()
        editor.rotateLeft()
        
        self.failUnless(len(editor.unredo.stack) == 3)
        
    def test_getCurrentImageData(self):
        # Not sure how to test this...
        pass
        
    def test_getCurrentImageInfo(self):
        editor = self.getImageEditorAdapter()
        info = editor.getCurrentImageInfo()
        
        self.failUnless(info['sizeformatted'] == "Size: 25kb")
        self.failUnless(info['width'] == 461)
        self.failUnless(info['height'] == 614)
        self.failUnless(info['size'] == 25)
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestImageEdits))
    suite.addTest(makeSuite(TestAdapter))
    
    return suite


import unittest
from Products.ImageEditor.tests.base import ImageEditorTestCase
from Products.CMFCore.utils import getToolByName
from plone.app.viewletmanager.interfaces import IViewletSettingsStorage
from zope.component import getUtility

class TestSetup(ImageEditorTestCase):
    """
    
    """
    
    def test_jquery_css_added(self):
        pcss = getToolByName(self.portal, 'portal_css')
        self.failUnless('++resource++jquery.ui.all.css' in [css.getId() for css in pcss.getResources()])

    def test_editor_css_added(self):
        pcss = getToolByName(self.portal, 'portal_css')
        self.failUnless('++resource++imageeditor-style.css' in [css.getId() for css in pcss.getResources()])

    def test_kss_added(self):
        kcss = getToolByName(self.portal, 'portal_kss')
        self.failUnless('++resource++imageeditor.kss' in [kss.getId() for kss in kcss.getResources()])
        
    def test_js_added(self):
        pjavascripts = getToolByName(self.portal, 'portal_javascripts')
        self.failUnless('++resource++jquery-1.2.6.js' in [js.getId() for js in pjavascripts.getResources()])
        self.failUnless('++resource++jquery-ui-imageeditor.min.js' in [js.getId() for js in pjavascripts.getResources()])
        self.failUnless('++resource++jquery.imgareaselect-0.4.2.min.js' in [js.getId() for js in pjavascripts.getResources()])
        self.failUnless('++resource++imageeditor.js' in [js.getId() for js in pjavascripts.getResources()])
        
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestSetup))
    return suite


from zope.interface import implements
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
import random
from time import gmtime, strftime
from Products.ImageEditor.interfaces.imageeditor import IImageEditorAdapter
from PIL import Image

class Edit(BrowserView):

    template = ViewPageTemplateFile('imageeditor.pt')

    def __init__(self, *args, **kwargs):
        super(BrowserView, self).__init__(*args, **kwargs)
        
        self.imageeditor = IImageEditorAdapter(self.context)
        
        # always start with new image
        # not sure if this is desired or not 
        # this way, old edits are removed
        self.imageeditor.clearEdits()

    def __call__(self):        
        #reset on each visit to page
        return self.template()
        
    def image_url(self):
        """
        This is used because sometimes browsers cache images that may have been edited
        """
        return self.context.absolute_url() + "?" + str(random.randint(0, 1000000))

    def canCompress(self):
        #cannot compress png files
        return str(self.imageeditor.getCurrentImage().format != "PNG")

    def getSize(self):
        return self.imageeditor.getCurrentImageInfo()['sizeformatted']

class ShowCurrentEdit(BrowserView):
    """
    Just a browser view to get latest edited image
    """
    def __init__(self, *args, **kwargs):
        super(BrowserView, self).__init__(*args, **kwargs)
        self.imageeditor = IImageEditorAdapter(self.context)

    def __call__(self):
        
        resp = self.request.response
        imagedata = self.imageeditor.getCurrentImageData()
        resp.setHeader('Content-Type', 'image/jpeg')
        resp.setHeader('Content-Length', len(imagedata))
        resp.setHeader('Last-Modified', strftime('%a, %d %b %Y %H:%M:%S +0000', gmtime()))
        resp.write(imagedata)
        return ''
    



def between(one, two):
    def con(val):
        assert val >= one and val <= two, "Value is not between %s and %s" % (one, two)
        
    return con
    
def precondition(precondition, use_conditions=True):
    """
    copied decorator from http://wiki.python.org/moin/PythonDecoratorLibrary#head-e1a9273f88f2fc4e39faf297279f4d1dc8b2ae66
    """
    return conditions(precondition, None, use_conditions)
 
def postcondition(postcondition, use_conditions=True):
    """
    copied decorator from http://wiki.python.org/moin/PythonDecoratorLibrary#head-e1a9273f88f2fc4e39faf297279f4d1dc8b2ae66
    """
    return conditions(None, postcondition, use_conditions)
 
class conditions(object):
    """
    copied decorator from http://wiki.python.org/moin/PythonDecoratorLibrary#head-e1a9273f88f2fc4e39faf297279f4d1dc8b2ae66
    """
    __slots__ = ('__precondition', '__postcondition')
    
    def __init__(self, pre, post, use_conditions=True):
        if not use_conditions:
            pre, post = None, None

        self.__precondition  = pre
        self.__postcondition = post

    def __call__(self, function):
        # combine recursive wrappers (@precondition + @postcondition == @conditions)
        pres  = set((self.__precondition,))
        posts = set((self.__postcondition,))
        
        # unwrap function, collect distinct pre-/post conditions
        while type(function) is FunctionWrapper:
            pres.add(function._pre)
            posts.add(function._post)
            function = function._func

        # filter out None conditions and build pairs of pre- and postconditions
        conditions = map(None, filter(None, pres), filter(None, posts))
        
        # add a wrapper for each pair (note that 'conditions' may be empty)
        for pre, post in conditions:
            function = FunctionWrapper(pre, post, function)
            
        return function
        
class FunctionWrapper(object):
    """
    copied decorator from http://wiki.python.org/moin/PythonDecoratorLibrary#head-e1a9273f88f2fc4e39faf297279f4d1dc8b2ae66
    """
    def __init__(self, precondition, postcondition, function):
        self._pre  = precondition
        self._post = postcondition
        self._func = function

    def __call__(self, *args, **kwargs):
        precondition  = self._pre
        postcondition = self._post

        if precondition:
            precondition(*args, **kwargs)
        result = self._func(*args, **kwargs)
        if postcondition:
            postcondition(result, *args, **kwargs)
        return result


import unittest
from Products.ImageEditor.tests.base import ImageEditorTestCase
from Products.CMFCore.utils import getToolByName
from plone.app.viewletmanager.interfaces import IViewletSettingsStorage
from zope.component import getUtility
from Products.ImageEditor.adapters.unredostack import UnredoStack
from cStringIO import StringIO

class TestUnredoStack(ImageEditorTestCase):
    """
    
    """
    
    def getUnredoAdapter(self):
        id = self.portal.invokeFactory(type_name="Image", id="testimage")
        image = self.portal['testimage']
        image.setTitle('test')
        
        im = self.getOriginal()
        imageData = StringIO()
        im.save(imageData, im.format)
        image.setImage(imageData.getvalue())
        
        return UnredoStack(image)
        
    def test_should_get_current_item_on_stack(self):
        unredo = self.getUnredoAdapter()
        
        self.failUnless(unredo.getCurrent() == unredo.image.data)
        
    def test_should_add_to_stack(self):
        editor = self.getImageEditorAdapter()
        
        editor.dropshadow()
        
        #starts from pos 0
        self.failUnless(editor.unredo.pos == 1)
        
    def test_should_be_able_to_undo(self):
        editor = self.getImageEditorAdapter()
        
        editor.dropshadow()
        
        self.failUnless(editor.unredo.canUndo())
        
    def test_should_not_be_able_undo(self):
        editor = self.getImageEditorAdapter()
        
        self.failUnless(not editor.unredo.canUndo())
        
    def test_undo_should_go_back_to_previous_edit(self):
        editor = self.getImageEditorAdapter()
        editor.dropshadow()
        editor.unredo.undo()
        self.failUnless(editor.unredo.getCurrent() == editor.image.data)
        
    def test_should_redo(self):
        editor = self.getImageEditorAdapter()
        editor.dropshadow()
        current_data = editor.unredo.getCurrent()
        editor.unredo.undo()
        editor.unredo.redo()
        self.failUnless(editor.unredo.getCurrent() == current_data)
        
    def test_canRedo_evaluates_correctly(self):
        editor = self.getImageEditorAdapter()
        editor.dropshadow()
        current_data = editor.unredo.getCurrent()
        editor.unredo.undo()
        self.failUnless(editor.unredo.canRedo())
        
    def test_canRedo_should_not_work(self):
        editor = self.getImageEditorAdapter()
        editor.dropshadow()
        current_data = editor.unredo.getCurrent()
        self.failUnless(not editor.unredo.canRedo())
        
    def test_clearStack_should_go_back_to_original_image(self):
        editor = self.getImageEditorAdapter()
        editor.dropshadow()
        editor.dropshadow()
        editor.dropshadow()
        editor.dropshadow()
        editor.unredo.clearStack()
        self.failUnless(editor.unredo.getCurrent() == editor.image.data)
        
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestUnredoStack))
    return suite


from zope.interface import Interface, Attribute

class IImageEditorAdapter(Interface):
     
    unredo = Attribute("""The unredo stack to handle undo and redo""")
        
    def undo(self):
        """
        undo last edit
        """
        
    def redo(self):
        """
        redo edit
        """
        
    def clearEdits(self):
        """
        clear all edits from undo/redo
        """

    def saveEdit(self):
        """
        This method gets the current image in the unredo stack and saves it to the
        object.  It then saves the history of it and gets the version_message so
        the history has better info...
        """
    def getOriginalImage(self):
        """
        
        """

    def getImageData(self):
        """
        get current image data from stack
        """
       
    def setImage(self, value):
       """
       Set new image on stack
       """
       
    def imageInfo(self):
        """
        return information on the current image
        """
       
    def rotateLeft(self):
        """
        rotate image left
        """
        
    def rotateRight(self):
        """
        rotate image right
        """
        
    def flipOnVerticalAxis(self):
        """
        """
        
    def flipOnHorizontalAxis(self):
        """
        """
        
    def blur(self, amount):
        """
        blur image by amount
        """
        
    def compress(self, amount):
        """
        compress the image and convert it to jpg if necessary
        """
        
    def contrast(self, amount):
        """
        set contrast on an image
        """
        
    def brightness(self, amount):
        """
        set brightness on an image
        """
        
    def sharpen(self, amount):
        """
        sharpen an image
        """
    
    def resize(self, width, height):
        """
        resize current image
        """
        
    def crop(self, tlx, tly, brx, bry):
        """
        crop current image
        """
        
    def dropshadow(self):
       """
       add drop shadow to current image
       """


# Copyright (c) 2006-2007
# Authors: KSS Project Contributors (see docs/CREDITS.txt)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

from kss.core.kssview import CommandSet
from zope.interface import implements

from zope.interface import Interface

class IImageEditorCommands(Interface):
    """The commands"""

    def setImage(selector, url, canUndo, canRedo, canSave, size, width, height, withKssSetup='True'):
        """
        """

class ImageEditorCommands(CommandSet):
    implements(IImageEditorCommands)
        
    def setImage(self, selector, url, canUndo, canRedo, canSave, size, width, height, withKssSetup='True'):
        """ see interfaces.py """
        command = self.commands.addCommand('setImage', selector)
        data = command.addParam('url', url)
        data = command.addParam('canUndo', canUndo)
        data = command.addParam('canRedo', canRedo)
        data = command.addParam('canSave', canSave)
        data = command.addParam('size', size)
        data = command.addParam('width', width)
        data = command.addParam('height', height)
        data = command.addParam('withKssSetup', withKssSetup)


from plone.app.kss.interfaces import IPloneKSSView
from plone.app.kss.plonekssview import PloneKSSView
from kss.core import kssaction
from zope.interface import implements
from Products.ImageEditor.interfaces.imageeditor import IImageEditorAdapter
import random
from Products.CMFCore.utils import getToolByName

def _(s):
    """
    takes a string and returns the int that PIL likes
    """
    return int(float(s))


class ImageEditorKSS(PloneKSSView):
    implements(IPloneKSSView)

    def __init__(self, *args, **kwargs):
        super(PloneKSSView, self).__init__(*args, **kwargs)
        
        self.imageeditor = IImageEditorAdapter(self.context)

    def callSetImageCommand(self):
        """
        
        """
        imageCommands = self.getCommandSet('imageeditor')
        ksscore = self.getCommandSet('core')
        
        imageInfo = self.imageeditor.getCurrentImageInfo()
        
        imageCommands.setImage(
            ksscore.getSameNodeSelector(), 
            self.context.absolute_url() + "/currenteditedimage.jpg?" + str(random.randint(0, 1000000)),
            str(int(self.imageeditor.unredo.canUndo())),
            str(int(self.imageeditor.unredo.canRedo())),
            str(int(self.imageeditor.unredo.pos > 0)),
            imageInfo['sizeformatted'],
            str(imageInfo['width']),
            str(imageInfo['height'])
        )
        
    @kssaction
    def saveImageEdit(self):          
        #force versioning to kick in
        portal_repository = getToolByName(self.context, 'portal_repository')
        if portal_repository.isVersionable(self.context):
            portal_repository.save(self.context, comment = "saved from image editor")
        
        self.imageeditor.saveEdit()
        self.context.reindexObject() #stop image caching on browser
        
        self.callSetImageCommand()
    
    @kssaction 
    def cancelImageEdit(self):
        self.imageeditor.clearEdits()
        self.callSetImageCommand()
        
    @kssaction
    def redoImageEdit(self):
        self.imageeditor.redo()
        self.callSetImageCommand()

    @kssaction 
    def undoImageEdit(self):
        self.imageeditor.undo()
        self.callSetImageCommand()

    @kssaction
    def rotateImageLeft(self):
        self.imageeditor.rotateLeft()
        self.callSetImageCommand()
        
    @kssaction
    def rotateImageRight(self):
        self.imageeditor.rotateRight()
        self.callSetImageCommand()

    @kssaction
    def imageFlipOnVerticalAxis(self):
        self.imageeditor.flipOnVerticalAxis()
        self.callSetImageCommand()
        
    @kssaction
    def blur(self, amount):
        self.imageeditor.blur(_(amount))
        self.callSetImageCommand()
        
    @kssaction
    def compress(self, amount):
        self.imageeditor.compress(_(amount))
        self.callSetImageCommand()
        
    @kssaction
    def contrast(self, amount):
        self.imageeditor.contrast(float("." + amount)*2.0)
        self.callSetImageCommand()
        
    @kssaction
    def brightness(self, amount):
        self.imageeditor.brightness(float("." + amount)*2.0)
        self.callSetImageCommand()

    @kssaction
    def sharpen(self, amount):
        self.imageeditor.sharpen(float(amount))
        self.callSetImageCommand()
        
    @kssaction
    def imageFlipOnHorizontalAxis(self):
        self.imageeditor.flipOnHorizontalAxis()
        self.callSetImageCommand()

    @kssaction
    def imageResizeSave(self, width, height):
        self.imageeditor.resize(_(width), _(height))
        self.callSetImageCommand()
        
    @kssaction
    def imageCropSave(self, topLeftX, topLeftY, bottomRightX, bottomRightY):
        self.imageeditor.crop(_(topLeftX), _(topLeftY), _(bottomRightX), _(bottomRightY))
        self.callSetImageCommand()
        
    @kssaction
    def cropAndResize(self, topLeftX, topLeftY, bottomRightX, bottomRightY, width, height):
        self.imageeditor.resize(_(width), _(height))
        self.imageeditor.crop(_(topLeftX), _(topLeftY), _(bottomRightX), _(bottomRightY))
        self.callSetImageCommand()
        
    @kssaction
    def addDropShadow(self):
        self.imageeditor.dropshadow()
        self.callSetImageCommand()


import unittest

from zope.testing import doctestunit
from zope.component import testing
from Testing import ZopeTestCase as ztc

from Products.Five import zcml
from Products.Five import fiveconfigure
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import PloneSite
from Products.PloneTestCase.layer import onsetup

from zope.app import zapi
from zope.configuration import xmlconfig
from Products.ImageEditor.adapters.imageeditor import ImageEditorAdapter

from PIL import Image, ImageFilter, ImageEnhance
from cStringIO import StringIO

import sys
from PIL import ImageChops

BASE_DIR = ""

for path in sys.path:
    if 'Products.ImageEditor' in path:
        BASE_DIR = path

ztc.installProduct('Products.ImageEditor')
ptc.setupPloneSite(products=('Products.ImageEditor',))

class ImageEditorTestCase(ptc.PloneTestCase):
    """
    """
    
    def afterSetUp(self):
        self.setRoles(('Manager',))
    
    def getImageEditorAdapter(self):
        id = self.portal.invokeFactory(type_name="Image", id="testimage")
        image = self.portal['testimage']
        image.setTitle('test')
        
        im = self.getOriginal()
        imageData = StringIO()
        im.save(imageData, im.format)
        image.setImage(imageData.getvalue())
        
        return ImageEditorAdapter(image)
       
    def imagesEqual(self, comparedImageFileName, imageEditor):
        imageEditor.saveEdit()
        imageEditor.getCurrentImage().save(self.getImagePath(comparedImageFileName))
        
        imgfile = self.getImage(comparedImageFileName)
        
        image_diff = ImageChops.difference(imgfile, imageEditor.getCurrentImage()).getbbox()
        
        #self.failUnless(image_diff[0] == 0 and image_diff[1] == 0)
        
        #doesn't ever work...  Why aren't images the same??
        #self.failUnless(image_diff is None)
        
    def getOriginal(self):
        return self.getImage('original.jpg')
        
    def getImagePath(self, name):
        return '%s/Products/ImageEditor/tests/testimages/%s' % (BASE_DIR, name)
        
    def getImage(self, name):
        return Image.open(self.getImagePath(name))